from typing import List
from dacite import from_dict
from citybrain_platform.storage import data_types
from citybrain_platform.storage.client import StorageClient
import citybrain_platform

__all__ = ["Storage"]

class Storage():
    __CLIENT: StorageClient = None

    @classmethod
    def __client(cls) -> StorageClient:
        if getattr(cls, '__CLIENT', None) is None:
            cls.__CLIENT = StorageClient(base_url=citybrain_platform.api_baseurl, api_key=citybrain_platform.api_key)
        return cls.__CLIENT
    
    @classmethod
    def upload_file(cls, remote_path: str, local_file: str) -> bool:
        result = False
        with open(local_file) as f:
            result = cls.__client().file_upload(path=remote_path, body=f)
        return result
    
    @classmethod
    def delete_file(cls, remote_path: str) -> bool:
        return cls.__client().file_delete(path=remote_path)
    
    @classmethod
    def list_files(cls, prefix: str = "", direct_only: bool = False) -> List[data_types.FileListItem]:
        result = cls.__client().file_list(prefix=prefix, direct_only=direct_only)
        table_list = [from_dict(data_class=data_types.FileListItem, data=item) for item in result]
        return table_list
    
    @classmethod
    def download_file(cls, remote_path: str, local_file: str):
        response = cls.__client().file_download(path=remote_path)
        with open(local_file, "wb") as f:
            for l in response.iter_lines():
                f.write(l)
                f.write(b'\n')



from re import T
import time

from citybrain_platform import JobStatus
import citybrain_platform
from citybrain_platform import Column, ColumnType


citybrain_platform.api_key = "MTAwMDEkMTY5MjE1NTM0NyQxMDAwMQ"
# citybrain_platform.api_key = "MiQxNjkyMTU1Nzk3JGJxaXplaXRu"
citybrain_platform.api_baseurl = "http://localhost:8080/platform/"
# citybrain_platform.api_baseurl = "https://dev.citybrain.org/platform/"




res = citybrain_platform.Storage.delete_file(remote_path="fawda/fsaw/a.csv")
# print(res)

# res = citybrain_platform.Storage.list_files(prefix="", direct_only=False)
# print(res)

# res = citybrain_platform.Storage.upload_file(remote_path="fawda/faaw/as.csv", local_file="a.csv")
# print(res)


# citybrain_platform.Storage.download_file(remote_path="fawda/faw/a.csv", local_file="b.csv")




# columns = [
#     Column("col_str", ColumnType.STRING, "aa"),
#     Column("col_id", ColumnType.TIMESTAMP),
# ]

# partition_columns = [
#     Column("col_pt", ColumnType.INT, "ppt")
# ]

# citybrain_platform.Computing.create_job("select adad, froms dada")


# create table
# ok = citybrain_platform.Computing.create_table(name="mbt_test67", columns=columns, partition_columns=partition_columns, description="ssdad")
# print(ok)

# get table schema
# schema = citybrain_platform.Computing.get_table_schema(name="mbt_test4")
# print(schema)

# upload data to table
# result = citybrain_platform.Computing.upload_table_data(name="mbt_test2", append=True, csv_filepath="aa.csv", partition_key={"col_pt": "19"})
# print(result)

# truncate table
# result = citybrain_platform.Computing.truncate_table(name="mbt_test1", partition_key={"col_pt": "1"})
# print(result)

# drop table
# result = citybrain_platform.Computing.drop_table(name="mbt_test2")
# print(result)

# public table
# public_table_name = citybrain_platform.Computing.public_table(name="mbt_test1")
# print(public_table_name)

# get available table list
# result = citybrain_platform.Computing.list_tables()
# print(result)

# create sql job
# job_id = citybrain_platform.Computing.create_job(
#     sql="select osmid from osm_node where osmid between 80080000 and 80081000;",
#     worker_limit=100,
#     split_size=256
# )
# print(job_id)

# stop running job
# result = citybrain_platform.Computing.stop_job(job_id="fab7b329-aef9-4c96-a9ac-1b47b1468e79")
# print(result)

# get job status
# job_status = citybrain_platform.Computing.get_job_status(job_id="e5597f65-b12a-4c3f-bb36-6cd742ec9b44")
# print(job_status)


# job_id = citybrain_platform.Computing.create_job(sql="select * from eric.zhang.city_Oklahoma limit 10;")
# print(job_id)

# get job results
# citybrain_platform.Computing.get_job_results(job_id="dc9132b8-8697-4c1e-9e77-796eb0ec7584", filepath="a.csv")


# sqlresult = "select * from cesm_ass1_50year_0730 limit 1000"
# jobresult_id = citybrain_platform.Computing.create_job(sql=sqlresult)

# while True:
#     status = citybrain_platform.Computing.get_job_status(job_id=jobresult_id)
#     print(status.status)
#     if status.status == JobStatus.RUNNING:
#         print(status.progress)
#     if status.status == JobStatus.TERMINATED:
#         break
#     time.sleep(1)

# print("downloading result")
# citybrain_platform.Computing.get_job_results(job_id=jobresult_id, filepath="cesm_ass1_50year_0730.csv")

# citybrain_platform.Data.download(data_address="174B948F17421000", save_file="a.txt")



from uplink import Body, Consumer, Query, Field, get, inject, json, post, response_handler
from uplink.hooks import TransactionHook

class APIResponseException(Exception):
    pass

class ResponseStreamHook(TransactionHook):
    def audit_request(self, consumer, request_builder):
        request_builder.info["stream"] = True
        return super().audit_request(consumer, request_builder)

_response_stream_hook = ResponseStreamHook()

def handle_json_response(response):
    response.iter_lines()
    if response.headers["Content-Type"].startswith("application/json"):
        body = response.json()
        if body["code"] != 200:
            raise APIResponseException(body["message"])
        return body["data"]
    return response

@response_handler(handle_json_response)
class ComputingClient(Consumer):
    """A Python Client for the Citybrain Computing Platform API."""

    def __init__(self, base_url, api_key):
        super(ComputingClient, self).__init__(base_url=base_url)
        self.session.headers["api_key"] = api_key

    @get("computing/table/detail")
    def table_info(self, name: Query("name")):
        pass

    @post("computing/table/upload")
    def table_upload(self, name: Query("name"), append: Query("append"), partition_key: Query("partition_key"), body: Body):
        pass

    @json
    @post("computing/table/create")
    def table_create(self, name: Field("name"), columns: Field("columns"), partition_columns: Field("partition_columns"), description: Field("description")):
        pass

    @json
    @post("computing/table/truncate")
    def table_truncate(self, name: Field("name"), partition_key: Field("partition_key")):
        pass

    @json
    @post("computing/table/drop")
    def table_drop(self, name: Field("name")):
        pass

    @json
    @post("computing/table/public")
    def table_public(self, name: Field("name")):
        pass

    @get("computing/table/list")
    def table_list(self):
        pass

    @json
    @post("computing/job/submit")
    def job_submit(self, sql: Field("sql"), worker_limit: Field("worker_limit"), split_size: Field("split_size")):
        pass

    @json
    @post("computing/job/stop")
    def job_stop(self, job_id: Field("job_id")):
        pass

    @get("computing/job/status")
    def job_status(self, job_id: Query("job_id")):
        pass

    @inject(_response_stream_hook)
    @get("computing/job/result")
    def job_result(self, job_id: Query("job_id")):
        pass


import os as __os
from citybrain_platform.computing import Computing
from citybrain_platform.storage import Storage
from citybrain_platform.data import Data
from citybrain_platform.computing.data_types import ColumnType, Column, JobStatus

api_key = __os.getenv("CITYBRAIN_APIKEY")
api_baseurl = __os.getenv("CITYBRAIN_API_BASEURL", "https://www.citybrain.org/platform/")

__all__ = [
    "Computing",
    "Storage"
    "Data",
    "api_key",
    "api_baseurl",
    "ColumnType",
    "Column",
    "JobStatus"
]



from uplink import Consumer, Query, get, inject
from uplink.hooks import TransactionHook

class ResponseStreamHook(TransactionHook):
    def audit_request(self, consumer, request_builder):
        request_builder.info["stream"] = True
        return super().audit_request(consumer, request_builder)

_response_stream_hook = ResponseStreamHook()

class DataClient(Consumer):
    """A Python Client for the Citybrain Data Platform API."""

    def __init__(self, base_url, api_key):
        super(DataClient, self).__init__(base_url=base_url)
        self.session.headers["api_key"] = api_key

    @inject(_response_stream_hook)
    @get("data/download")
    def download(self, data_address: Query("data_address")):
        pass
